/*-
 * Copyright (C)2014 @BABOLO http://www.babolo.ru/
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * $Id: recobe.h,v 1.12 2014/09/18 12:25:07 babolo Exp $
 */

#ifndef  _RECOBE_H_
# define _RECOBE_H_

__BEGIN_DECLS

# define RECOBE_PRINTABLE  0x000100
# define RECOBE_PRINTATLE  0x000200
# define RECOBE_FULLCHECK  0x000400
# define RECOBE_PRELINERA  0x000800
# define RECOBE_REVERTIZE  0x010000
# define RECOBE_INTERTIZE  0x020000

# define RECOBE_NOTABLE    0
# define RECOBE_TEXT       1
# define RECOBE_G711       2
# define RECOBE_ETAT       3
# define RECOBE_DOMAX      4

typedef struct {
    u_int64_t domain : 16
            , src    : 24
            , dst    : 24
    ;
} recobe_conv;

typedef struct {
    const char         *n;                                  /* table code part name                  */
    const char       *(*s)[];                               /* code names                            */
    const char         *p;                                  /* printable                             */
} recobe_c_tnm;

typedef struct {
    const u_int16_t     cw;                                 /* value RECOBE_XXXX_L                   */
    const char         *nn;                                 /* domain name                           */
    const char         *pp;                                 /* table common part names               */
    const char         *st;                                 /* table name separator                  */
    const char         *sc;                                 /* recode name separator                 */
    const recobe_c_tnm *nm;                                 /* referenses to code descriptors        */
    int               (*tst)(const char *nm);               /*    nm  -1  */
    u_char           *(*ind)(recobe_conv cnv);              /*    cnv          */
    recobe_conv       (*get)(const char *name);             /*    name         */
} recobe_c_tdm;

typedef struct {
    const u_int16_t     mw;                                 /* value RECOBE_DOMAX                    */
    u_char           *(*get)(const char *name, int domain); /*    name         */
    const recobe_c_tdm *dm[];                               /* referenses to domain descriptors      */
} recobe_collection_t;

/*****************************************************************************************************/
/*                        =============== G.711 ===============                                      */
/* recobe_deG711A      G.711 A-law                                          */
/* recobe_deG711u      G.711 u-law                                          */
/* recobe_G711A          G.711 A-law                                            */
/* recobe_G711u          G.711 u-law                                            */
/* recobe_G711Atest   buf  l  1KHz c  fi (0..7)   G.711 A-law       */
/* recobe_G711utest   buf  l  1KHz c  fi (0..7)   G.711 u-law       */
                                                                                                   /**/
extern const int16_t             recobe_deG711A[256];                                              /**/
extern const int16_t             recobe_deG711u[256];                                              /**/
u_char    recobe_G711A        __P((int16_t sample));                                               /**/
u_char    recobe_G711u        __P((int16_t sample));                                               /**/
int       recobe_G711Atest    __P((int fi, u_char *buf, size_t l));                                /**/
int       recobe_G711utest    __P((int fi, u_char *buf, size_t l));                                /**/
                                                                                                   /**/
/*****************************************************************************************************/

/*****************************************************************************************************/
/* recobe_collection                                            */
                                                                                                   /**/
extern const recobe_collection_t recobe_collection;                                                /**/
                                                                                                   /**/
/*****************************************************************************************************/

/*****************************************************************************************************/
/*                                                                              */
extern const char recobe_nullprintable[256];                                                       /**/
extern const char recobe_koi8printable[256];                                                       /**/
/*****************************************************************************************************/

/*****************************************************************************************************/
/*   octets      .                                        */
/* 0 > len                                                       */
/* recobe_crc32    crc32 (IEEE 802.3)   octets  |len|        */
/* recobe_crc24    crc24 (RFC-2440)   octets  |len|          */
/* recobe_csum1        octets  |len|          */
/* recobe_load       octets  |len|            */
/* recobe_loadM     octets    mask                                            */
/* recobe_store    c     octets  |len|                   */
/* recobe_storeM   c     octets   mask                            */
/* recobe_swap        c   m                                           */
                                                                                                   /**/
u_int32_t recobe_crc32     __P((const u_char *octets, ssize_t len));                               /**/
u_int32_t recobe_crc24     __P((const u_char *octets, ssize_t len));                               /**/
u_int32_t recobe_csum1     __P((const u_char *octets, ssize_t len));                               /**/
u_int64_t recobe_load      __P((const u_char *octets, ssize_t len));                               /**/
u_int64_t recobe_loadM     __P((const u_char *octets, const char *mask));                          /**/
void      recobe_store     __P((u_char *octets, ssize_t len, u_int64_t c));                        /**/
void      recobe_storeM    __P((u_char *octets, const char *mask, u_int64_t c));                   /**/
u_int64_t recobe_swap      __P((int64_t c, u_int8_t m));                                           /**/
                                                                                                   /**/
/*****************************************************************************************************/

__END_DECLS

#endif /* _RECOBE_H_ */
